@@ -1,4 +1,7 @@
-module Database.Persist.Class.DeleteCascade
+{-# LANGUAGE ExplicitForAll #-}
+
+
+module Database.Persist.Class.DeleteCascade {-# DEPRECATED "The DeleteCascade module is deprecated. You can now set cascade behavior directly on entities in the quasiquoter." #-}
     ( DeleteCascade (..)
     , deleteCascadeWhere
     ) where
@@ -13,6 +16,8 @@ import Database.Persist.Class.PersistStore
 import Database.Persist.Class.PersistQuery
 import Database.Persist.Class.PersistEntity
 
+{-# DEPRECATED DeleteCascade "The DeleteCascade class is deprecated since you can now define cascade behavior directly on an entity." #-}
+
 -- | For combinations of backends and entities that support
 -- cascade-deletion. “Cascade-deletion” means that entries that depend on
 -- other entries to be deleted will be deleted as well.
@@ -23,8 +28,10 @@ class (PersistStoreWrite backend, PersistEntity record, BaseBackend backend ~ Pe
     -- entry.
     deleteCascade :: MonadIO m => Key record -> ReaderT backend m ()
 
+{-# DEPRECATED deleteCascadeWhere "This function is deprecated since you can set cascading delete behavior directly on the entity." #-}
+
 -- | Cascade-deletion of entries satisfying given filters.
-deleteCascadeWhere :: (MonadIO m, DeleteCascade record backend, PersistQueryWrite backend)
+deleteCascadeWhere :: forall record backend m. (MonadIO m, DeleteCascade record backend, PersistQueryWrite backend)
                    => [Filter record] -> ReaderT backend m ()
 deleteCascadeWhere filts = do
     srcRes <- selectKeysRes filts []
