@@ -1,23 +1,26 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE GADTs #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE PatternGuards, DataKinds, TypeOperators, UndecidableInstances, GeneralizedNewtypeDeriving #-}
 module Database.Persist.Class.PersistField
     ( PersistField (..)
     , SomePersistField (..)
     , getPersistMap
+    , OverflowNatural(..)
     ) where
 
 import Control.Arrow (second)
 import Control.Monad ((<=<))
+import Control.Applicative ((<|>))
 import qualified Data.Aeson as A
 import Data.ByteString.Char8 (ByteString, unpack, readInt)
 import qualified Data.ByteString.Lazy as L
 import Data.Fixed
 import Data.Int (Int8, Int16, Int32, Int64)
 import qualified Data.IntMap as IM
+import qualified Data.List.NonEmpty as NonEmpty
 import qualified Data.Map as M
-import Data.Monoid ((<>))
 import qualified Data.Set as S
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -30,6 +33,7 @@ import Data.Word (Word, Word8, Word16, Word32, Word64)
 import Numeric.Natural (Natural)
 import Text.Blaze.Html
 import Text.Blaze.Html.Renderer.Text (renderHtml)
+import GHC.TypeLits
 
 import Database.Persist.Types.Base
 
@@ -108,7 +112,7 @@ instance {-# OVERLAPPING #-} PersistField [Char] where
     fromPersistValue (PersistBool b) = Right $ Prelude.show b
     fromPersistValue (PersistList _) = Left $ T.pack "Cannot convert PersistList to String"
     fromPersistValue (PersistMap _) = Left $ T.pack "Cannot convert PersistMap to String"
-    fromPersistValue (PersistDbSpecific _) = Left $ T.pack "Cannot convert PersistDbSpecific to String. See the documentation of PersistDbSpecific for an example of using a custom database type with Persistent."
+    fromPersistValue (PersistLiteral_ _ _) = Left $ T.pack "Cannot convert PersistLiteral_ to String"
     fromPersistValue (PersistArray _) = Left $ T.pack "Cannot convert PersistArray to String"
     fromPersistValue (PersistObjectId _) = Left $ T.pack "Cannot convert PersistObjectId to String"
 #endif
@@ -303,18 +307,28 @@ instance PersistField UTCTime where
     fromPersistValue (PersistInt64 i)   = Right $ posixSecondsToUTCTime $ (/ (1000 * 1000 * 1000)) $ fromIntegral $ i
 #endif
     fromPersistValue x@(PersistText t)  =
-        case reads $ T.unpack t of
-            (d, _):_ -> Right d
-            _ ->
-                case parse8601 $ T.unpack t of
+        let s = T.unpack t
+        in
+          case NonEmpty.nonEmpty (reads s) of
+            Nothing ->
+                case parse8601 s <|> parsePretty s of
                     Nothing -> Left $ fromPersistValueParseError "UTCTime" x
                     Just x' -> Right x'
+            Just matches ->
+                -- The 'Read UTCTime' instance in newer versions of 'time' is
+                -- more flexible when parsing UTCTime strings and will return
+                -- UTCTimes with different microsecond parsings. The last result
+                -- here contains the parsed UTCTime with as much microsecond
+                -- precision parsed as posssible.
+                Right $ fst $ NonEmpty.last matches
       where
 #if MIN_VERSION_time(1,5,0)
-        parse8601 = parseTimeM True defaultTimeLocale "%FT%T%Q"
+        parseTime' = parseTimeM True defaultTimeLocale
 #else
-        parse8601 = parseTime defaultTimeLocale "%FT%T%Q"
+        parseTime' = parseTime defaultTimeLocale
 #endif
+        parse8601 = parseTime' "%FT%T%Q"
+        parsePretty = parseTime' "%F %T%Q"
     fromPersistValue x@(PersistByteString s) =
         case reads $ unpack s of
             (d, _):_ -> Right d
@@ -322,11 +336,43 @@ instance PersistField UTCTime where
 
     fromPersistValue x = Left $ fromPersistValueError "UTCTime" "time, integer, string, or bytestring" x
 
-instance PersistField Natural where
-  toPersistValue = (toPersistValue :: Int64 -> PersistValue) . fromIntegral
+-- | Prior to @persistent-2.11.0@, we provided an instance of
+-- 'PersistField' for the 'Natural' type. This was in error, because
+-- 'Natural' represents an infinite value, and databases don't have
+-- reasonable types for this.
+--
+-- The instance for 'Natural' used the 'Int64' underlying type, which will
+-- cause underflow and overflow errors. This type has the exact same code
+-- in the instances, and will work seamlessly.
+--
+-- A more appropriate type for this is the 'Word' series of types from
+-- "Data.Word". These have a bounded size, are guaranteed to be
+-- non-negative, and are quite efficient for the database to store.
+--
+-- @since 2.11.0
+newtype OverflowNatural = OverflowNatural { unOverflowNatural :: Natural }
+    deriving (Eq, Show, Ord, Num)
+
+instance
+  TypeError
+    ( 'Text "The instance of PersistField for the Natural type was removed."
+    ':$$: 'Text "Please see the documentation for OverflowNatural if you want to "
+    ':$$: 'Text "continue using the old behavior or want to see documentation on "
+    ':$$: 'Text "why the instance was removed."
+    ':$$: 'Text ""
+    ':$$: 'Text "This error instance will be removed in a future release."
+    )
+  =>
+    PersistField Natural
+  where
+    toPersistValue = undefined
+    fromPersistValue = undefined
+
+instance PersistField OverflowNatural where
+  toPersistValue = (toPersistValue :: Int64 -> PersistValue) . fromIntegral . unOverflowNatural
   fromPersistValue x = case (fromPersistValue x :: Either Text Int64) of
-    Left err -> Left $ T.replace "Int64" "Natural" err
-    Right int -> Right $ fromIntegral int -- TODO use bimap?
+    Left err -> Left $ T.replace "Int64" "OverflowNatural" err
+    Right int -> Right $ OverflowNatural $ fromIntegral int -- TODO use bimap?
 
 instance PersistField a => PersistField (Maybe a) where
     toPersistValue Nothing = PersistNull
@@ -406,7 +452,7 @@ getPersistMap (PersistByteString bs)
 getPersistMap PersistNull = Right []
 getPersistMap x = Left $ fromPersistValueError "[(Text, PersistValue)]" "map, string, bytestring or null" x
 
-data SomePersistField = forall a. PersistField a => SomePersistField a
+data SomePersistField = forall a. (PersistField a) => SomePersistField a
 instance PersistField SomePersistField where
     toPersistValue (SomePersistField a) = toPersistValue a
     fromPersistValue x = fmap SomePersistField (fromPersistValue x :: Either Text Text)
