@@ -1,6 +1,8 @@
 {-# LANGUAGE ConstraintKinds #-}
+{-# LANGUAGE ExplicitForAll #-}
 module Database.Persist.Class.PersistStore
     ( HasPersistBackend (..)
+    , withBaseBackend
     , IsPersistBackend (..)
     , PersistRecordBackend
     , liftPersist
@@ -16,12 +18,13 @@ module Database.Persist.Class.PersistStore
     , insertRecord
     , ToBackendKey(..)
     , BackendCompatible(..)
+    , withCompatibleBackend
     ) where
 
 import Control.Exception (throwIO)
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Control.Monad.Reader (MonadReader (ask), runReaderT)
-import Control.Monad.Trans.Reader (ReaderT)
+import Control.Monad.Trans.Reader (ReaderT, withReaderT)
 import qualified Data.Aeson as A
 import Data.Map (Map)
 import qualified Data.Map as Map
@@ -42,6 +45,16 @@ import Database.Persist.Types
 class HasPersistBackend backend where
     type BaseBackend backend
     persistBackend :: backend -> BaseBackend backend
+
+-- | Run a query against a larger backend by plucking out @BaseBackend backend@
+--
+-- This is a helper for reusing existing queries when expanding the backend type.
+--
+-- @since 2.12.0
+withBaseBackend :: (HasPersistBackend backend)
+                => ReaderT (BaseBackend backend) m a -> ReaderT backend m a
+withBaseBackend = withReaderT persistBackend
+
 -- | Class which witnesses that @backend@ is essentially the same as @BaseBackend backend@.
 -- That is, they're isomorphic and @backend@ is just some wrapper over @BaseBackend backend@.
 class (HasPersistBackend backend) => IsPersistBackend backend where
@@ -50,6 +63,10 @@ class (HasPersistBackend backend) => IsPersistBackend backend where
     -- to accidentally run a write query against a read-only database.
     mkPersistBackend :: BaseBackend backend -> backend
 
+-- NB: there is a deliberate *lack* of an equivalent to 'withBaseBackend' for
+-- 'IsPersistentBackend'. We don't want it to be easy for the user to construct
+-- a backend when they're not meant to.
+
 -- | This class witnesses that two backend are compatible, and that you can
 -- convert from the @sub@ backend into the @sup@ backend. This is similar
 -- to the 'HasPersistBackend' and 'IsPersistBackend' classes, but where you
@@ -87,13 +104,23 @@ class (HasPersistBackend backend) => IsPersistBackend backend where
 --
 -- -- after:
 -- asdf' :: 'BackendCompatible' SqlBackend backend => ReaderT backend m ()
--- asdf' = withReaderT 'projectBackend' asdf
+-- asdf' = 'withCompatibleBackend' asdf
 -- @
 --
 -- @since 2.7.1
 class BackendCompatible sup sub where
     projectBackend :: sub -> sup
 
+-- | Run a query against a compatible backend, by projecting the backend
+--
+-- This is a helper for using queries which run against a specific backend type
+-- that your backend is compatible with.
+--
+-- @since 2.12.0
+withCompatibleBackend :: (BackendCompatible sup sub)
+                      => ReaderT sup m a -> ReaderT sub m a
+withCompatibleBackend = withReaderT projectBackend
+
 -- | A convenient alias for common type signatures
 type PersistRecordBackend record backend = (PersistEntity record, PersistEntityBackend record ~ BaseBackend backend)
 
@@ -147,7 +174,7 @@ class
     -- > +------+-----+
     -- > | SPJ  |  40 |
     -- > +------+-----+
-    get :: (MonadIO m, PersistRecordBackend record backend)
+    get :: forall record m. (MonadIO m, PersistRecordBackend record backend)
         => Key record -> ReaderT backend m (Maybe record)
 
     -- | Get many records by their respective identifiers, if available.
@@ -173,7 +200,7 @@ class
     -- > |  2 | Simon |  41 |
     -- > +----+-------+-----+
     getMany
-        :: (MonadIO m, PersistRecordBackend record backend)
+        :: forall record m. (MonadIO m, PersistRecordBackend record backend)
         => [Key record] -> ReaderT backend m (Map (Key record) record)
     getMany [] = return Map.empty
     getMany ks = do
@@ -212,7 +239,7 @@ class
     -- > +-----+------+-----+
     -- > |3    |John  |30   |
     -- > +-----+------+-----+
-    insert :: (MonadIO m, PersistRecordBackend record backend)
+    insert :: forall record m. (MonadIO m, PersistRecordBackend record backend)
            => record -> ReaderT backend m (Key record)
 
     -- | Same as 'insert', but doesn't return a @Key@.
@@ -235,7 +262,7 @@ class
     -- > +-----+------+-----+
     -- > |3    |John  |30   |
     -- > +-----+------+-----+
-    insert_ :: (MonadIO m, PersistRecordBackend record backend)
+    insert_ :: forall record m. (MonadIO m, PersistRecordBackend record backend)
             => record -> ReaderT backend m ()
     insert_ record = insert record >> return ()
 
@@ -273,7 +300,7 @@ class
     -- > +-----+------+-----+
     -- > |5    |Jane  |20   |
     -- > +-----+------+-----+
-    insertMany :: (MonadIO m, PersistRecordBackend record backend)
+    insertMany :: forall record m. (MonadIO m, PersistRecordBackend record backend)
                => [record] -> ReaderT backend m [Key record]
     insertMany = mapM insert
 
@@ -304,7 +331,7 @@ class
     -- > +-----+------+-----+
     -- > |5    |Jane  |20   |
     -- > +-----+------+-----+
-    insertMany_ :: (MonadIO m, PersistRecordBackend record backend)
+    insertMany_ :: forall record m. (MonadIO m, PersistRecordBackend record backend)
                 => [record] -> ReaderT backend m ()
     insertMany_ x = insertMany x >> return ()
 
@@ -336,7 +363,7 @@ class
     -- > +-----+------+-----+
     -- > |4    |Eva   |38   |
     -- > +-----+------+-----+
-    insertEntityMany :: (MonadIO m, PersistRecordBackend record backend)
+    insertEntityMany :: forall record m. (MonadIO m, PersistRecordBackend record backend)
                      => [Entity record] -> ReaderT backend m ()
     insertEntityMany = mapM_ (\(Entity k record) -> insertKey k record)
 
@@ -361,7 +388,7 @@ class
     -- > +-----+------+-----+
     -- > |3    |Alice |20   |
     -- > +-----+------+-----+
-    insertKey :: (MonadIO m, PersistRecordBackend record backend)
+    insertKey :: forall record m. (MonadIO m, PersistRecordBackend record backend)
               => Key record -> record -> ReaderT backend m ()
 
     -- | Put the record in the database with the given key.
@@ -424,7 +451,7 @@ class
     -- > +-----+------+-----+
     -- > |3    |X     |999  |
     -- > +-----+------+-----+
-    repsert :: (MonadIO m, PersistRecordBackend record backend)
+    repsert :: forall record m. (MonadIO m, PersistRecordBackend record backend)
             => Key record -> record -> ReaderT backend m ()
 
     -- | Put many entities into the database.
@@ -455,7 +482,7 @@ class
     -- > |999  |Mr. X           |999      |
     -- > +-----+----------------+---------+
     repsertMany
-        :: (MonadIO m, PersistRecordBackend record backend)
+        :: forall record m. (MonadIO m, PersistRecordBackend record backend)
         => [(Key record, record)] -> ReaderT backend m ()
     repsertMany = mapM_ (uncurry repsert)
 
@@ -480,7 +507,7 @@ class
     -- > +-----+------+-----+
     -- > |2    |Simon |41   |
     -- > +-----+------+-----+
-    replace :: (MonadIO m, PersistRecordBackend record backend)
+    replace :: forall record m. (MonadIO m, PersistRecordBackend record backend)
             => Key record -> record -> ReaderT backend m ()
 
     -- | Delete a specific record by identifier. Does nothing if record does
@@ -500,7 +527,7 @@ class
     -- > +-----+------+-----+
     -- > |2    |Simon |41   |
     -- > +-----+------+-----+
-    delete :: (MonadIO m, PersistRecordBackend record backend)
+    delete :: forall record m. (MonadIO m, PersistRecordBackend record backend)
            => Key record -> ReaderT backend m ()
 
     -- | Update individual fields on a specific record.
@@ -523,7 +550,7 @@ class
     -- > +-----+------+-----+
     -- > |2    |Simon |41   |
     -- > +-----+------+-----+
-    update :: (MonadIO m, PersistRecordBackend record backend)
+    update :: forall record m. (MonadIO m, PersistRecordBackend record backend)
            => Key record -> [Update record] -> ReaderT backend m ()
 
     -- | Update individual fields on a specific record, and retrieve the
@@ -550,7 +577,7 @@ class
     -- > +-----+------+-----+
     -- > |2    |Simon |41   |
     -- > +-----+------+-----+
-    updateGet :: (MonadIO m, PersistRecordBackend record backend)
+    updateGet :: forall record m. (MonadIO m, PersistRecordBackend record backend)
               => Key record -> [Update record] -> ReaderT backend m record
     updateGet key ups = do
         update key ups
@@ -583,10 +610,11 @@ class
 -- mrx <- getJustUnknown
 --
 -- This just throws an error.
-getJust :: ( PersistStoreRead backend
-           , PersistRecordBackend record backend
-           , MonadIO m
-           ) => Key record -> ReaderT backend m record
+getJust :: forall record backend m.
+  ( PersistStoreRead backend
+  , PersistRecordBackend record backend
+  , MonadIO m)
+  => Key record -> ReaderT backend m record
 getJust key = get key >>= maybe
   (liftIO $ throwIO $ PersistForeignConstraintUnmet $ T.pack $ show key)
   return
@@ -611,11 +639,11 @@ getJust key = get key >>= maybe
 -- > +----+------+-----+
 -- > |  1 | SPJ  |  40 |
 -- > +----+------+-----+
-getJustEntity
-  :: (PersistEntityBackend record ~ BaseBackend backend
-     ,MonadIO m
-     ,PersistEntity record
-     ,PersistStoreRead backend)
+getJustEntity :: forall record backend m.
+  ( PersistEntityBackend record ~ BaseBackend backend
+  , MonadIO m
+  , PersistEntity record
+  , PersistStoreRead backend)
   => Key record -> ReaderT backend m (Entity record)
 getJustEntity key = do
   record <- getJust key
@@ -628,7 +656,7 @@ getJustEntity key = do
 -- | Curry this to make a convenience function that loads an associated model.
 --
 -- > foreign = belongsTo foreignId
-belongsTo ::
+belongsTo :: forall ent1 ent2 backend m.
   ( PersistStoreRead backend
   , PersistEntity ent1
   , PersistRecordBackend ent2 backend
@@ -639,7 +667,7 @@ belongsTo foreignKeyField model = case foreignKeyField model of
     Just f -> get f
 
 -- | Same as 'belongsTo', but uses @getJust@ and therefore is similarly unsafe.
-belongsToJust ::
+belongsToJust :: forall ent1 ent2 backend m.
   ( PersistStoreRead backend
   , PersistEntity ent1
   , PersistRecordBackend ent2 backend
@@ -670,7 +698,7 @@ belongsToJust getForeignKey model = getJust $ getForeignKey model
 -- > +----+---------+-----+
 -- > |  3 | Haskell |  81 |
 -- > +----+---------+-----+
-insertEntity ::
+insertEntity :: forall e backend m.
     ( PersistStoreWrite backend
     , PersistRecordBackend e backend
     , MonadIO m
@@ -697,7 +725,7 @@ insertEntity e = do
 -- > +----+------+-----+
 -- > |  1 | SPJ  |  40 |
 -- > +----+------+-----+
-getEntity ::
+getEntity :: forall e backend m.
     ( PersistStoreRead backend
     , PersistRecordBackend e backend
     , MonadIO m
@@ -731,7 +759,7 @@ getEntity key = do
 -- > |3    |Dave  |50   |
 -- > +-----+------+-----+
 insertRecord
-  :: (PersistEntityBackend record ~ BaseBackend backend
+  :: forall record backend m. (PersistEntityBackend record ~ BaseBackend backend
      ,PersistEntity record
      ,MonadIO m
      ,PersistStoreWrite backend)
